#include "jsweb.h"
#include "global.h"

#pragma mark --- Global Functions ---

static JSBool
CollectGarbage(JSContext* cx, JSObject* obj, uintN argc, jsval* argv, jsval* rval)
{
    printf("*** gc begin\n");
    JS_GC(cx);
    printf("*** gc complete\n");
    return JS_TRUE;
}

static JSBool
Print(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
    uintN i;
    JSString *str;

    for (i = 0; i < argc; i++) {
        str = JS_ValueToString(cx, argv[i]);
        if (!str)
            return JS_FALSE;
        printf("%s%s", i ? " " : "", JS_GetStringBytes(str));
    }
    printf("\n");
    return JS_TRUE;
}

static JSBool
Version(JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
{
    if (argc > 0 && JSVAL_IS_INT(argv[0]))
        *rval = INT_TO_JSVAL(JS_SetVersion(cx, (JSVersion) JSVAL_TO_INT(argv[0])));
    else
        *rval = INT_TO_JSVAL(JS_GetVersion(cx));
    return JS_TRUE;
}

/* We use a mix of JS_FS and JS_FN to test both kinds of natives. */
static JSFunctionSpec shell_functions[] = {
    JS_FS("print", Print, 0,0,0),
    JS_FS("version", Version, 0, 0, 0),
    JS_FS("gc", CollectGarbage, 0, 0, 0),
    JS_FS_END
};

#pragma mark --- Global Object Methods ----

JSClass global_class = {
    "global", JSCLASS_GLOBAL_FLAGS,
    JS_PropertyStub,  JS_PropertyStub,
    JS_PropertyStub,  JS_PropertyStub,
    JS_EnumerateStub, JS_ResolveStub,
    JS_ConvertStub,   JS_FinalizeStub,
    JSCLASS_NO_OPTIONAL_MEMBERS
};

JSObject* setupGlobal(JSContext* cx, JSObject* scope) {
    JSObject* global = JS_NewObject(cx, &global_class, NULL, NULL);
    if (!global)
        return NULL;
    if (!JS_InitStandardClasses(cx, global))
        return NULL;
    if (!JS_DefineFunctions(cx, global, shell_functions))
        return NULL;

    return global;
}
